function [G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,NumLead,StateVariablePositions,StateEquationPositions,StateShockPositions,T_ZLB,RtildePosition,REqPosition,AIMTolerance,OrderQZ)
% ZLBKleinSolver: Solves a linear rational expectations model with anticipated shocks and subject to a zero lower bound
%                 with Paul Klein's generalized Schur form based algorithm.
%
% USAGE:
%
%       [G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,NumLead,StateVariablePositions,StateEquationPositions, ...
%                                       StateShockPositions,T_ZLB,RtildePosition,REqPosition,AIMTolerance,OrderQZ)
%
% REQUIRED INPUT:  ModelParameters (structure) contains numeric values for the model parameters.
%
%                  NumLead (integer) with the number of leads.
%
%                  StateVariablePositions (vector) with r<=N containing the positions of the actual state
%                                         variables. AiM will typically stack the state variables and the
%                                         state shocks given that the shocks have been formulated as
%                                         equations!
%
%                  StateEquationPositions (vector) of dimension r with the positions for the state equations.
%
%                  StateShockPositions (vector) with q<=N containing the positions of the state shocks.
%
%                  T_ZLB (integer) with the length of the sample over which the zero lower bound may be
%                                         binding.
%
%                  RtildePosition (integer) with the position of the monetary policy rate among the state variables.
%
%                  REqPosition (integer) with the position of the monetary policy rule among the state
%                                         equations.
%
%                  AIMTolerance (positive integer) that determines the tolerance for the DSGE model solver.
%
%                  OrderQZ (boolean) which takes the value 1 if "ordqz" is a built-in matlab function
%                                         and 0 otherwise.
%
% REQUIRED OUTPUT: G (matrix) of dimension (T+q+r+2)x(T+q+r+2) with the values on the lagged state variables and
%                                         anticipated shocks in the solution of the model.
%
%                  C1 (matrix) of dimension (T+q+r+2)x(T+q+1) with the values on the current structural shocks
%                                         and innovations to the anticipated shocks in the solution of the
%                                         model.
%
%                  P (matrix) of dimension NumLead*(r+1)x(T+q+r+2) that maps the forward looking variables to the
%                                         predetermined variables and the anticipated shocks.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not. mcode is
%                                         1 when unique, 0 if not computed, and various other values when
%                                         there are problems with the solution. Specifically:
%
%                                              mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                              numbers.
%                                              mcode = -1:     No stable solution.
%                                              mcode = -2:     Too many large generalized eigenvalues.
%                                              mcode = -3:     Too few large generalized eigenvalues.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2015 European Central Bank.
%
%                       First version: August 22, 2014.
%                        This version: January 13, 2015.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-08-2014: Continued work on the function from the setting up of the A, B and C matrices.
%
% * 26-08-2014: Added the input variable "REqPosition".
%
% * 27-08-2014: Fixed a bug when computing the P matrix that projects the forward-looking variables
%               using the predetermined variables.
%
% * 01-09-2014: Finished the part where multiple leads is covered.
%
% * 02-09-2014: Updated the documentation.
%
% * 11-09-2014: Updated the documentation. Changed the input variable "RhatPosition" to "RtildePosition".
%
% * 13-01-2015: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
G = [];
C1 = [];
P = [];
mcode = 0;
%
condn = 10^(-AIMTolerance-2);
%
% Construct structural coefficient matrices.
%
[GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
if isreal(GMatrix)==0;
   mcode = 8;
   return;
end;
if isreal(HMatrix)==0;
   mcode = 8;
   return;
end;
[rh,ch] = size(HMatrix);
[rg,cg] = size(GMatrix);
H = zeros(rh,ch);
H(1:rg,1:cg) = GMatrix;
H = H+HMatrix;
Hlag = H(:,1:rh);
Hexp = H(:,(2*rh)+1:(2+NumLead)*rh);
H = H(:,rh+1:2*rh);
StateVarPos = StateVariablePositions;
if NumLead>1;
   for j=2:NumLead;
      StateVarPos = [StateVarPos StateVariablePositions+(rh*(j-1))];
   end;
end;
Hlag = Hlag(StateEquationPositions,StateVariablePositions);
Hexp = Hexp(StateEquationPositions,StateVarPos);
D = -H(StateEquationPositions,StateShockPositions);
H = H(StateEquationPositions,StateVariablePositions);
[k,l] = size(Hexp);
q = length(StateShockPositions);
%
% transform the system to Klein's A and B matrices
%
e_T1 = zeros(T_ZLB+1,1);
e_T1(1) = 1;
e_ps = zeros(k,1);
e_ps(RtildePosition) = 1;
e_pm = zeros(k,1);
e_pm(REqPosition) = 1;
Ik = eye(k);
Kps = Ik(:,setdiff((1:k),RtildePosition));
Kpm = Ik(:,setdiff((1:k),REqPosition));
%
% new code starts here
%
T = T_ZLB;
if T>1;
   M = [zeros(T,1) eye(T);zeros(1,T+1)];
else;
   M = 0;
end;
%
% define matrices from the non-monetary policy equations
%
Hmlag = Kpm'*Hlag;
Hm = Kpm'*H;
hmlag = e_pm'*Hlag*Kps*Kps';
hm = e_pm'*H*Kps*Kps';
hslag = e_pm'*Hlag*e_ps;
hs = e_pm'*H*e_ps;
Dm = Kpm'*D;
dm = e_pm'*D;
if NumLead==1;
   Hmexp = Kpm'*Hexp;
   hmexp = e_pm'*Hexp*Kps*Kps';
   hsexp = e_pm'*Hexp*e_ps;
   %
   % define A, B and C
   %
   A = [eye(T+2+k+q) zeros(T+2+k+q,k+1);zeros(k-1,T+2+k+q) Hmexp zeros(k-1,1);zeros(1,T+2+k+q) hmexp hsexp;zeros(1,T+3+q+(2*k))];
   B = [zeros(q,T+3+q+(2*k));zeros(k,T+2+k+q) eye(k) zeros(k,1);zeros(1,T+2+(2*k)+q) 1;zeros(T+1,k+q+1) M zeros(T+1,k+1);Dm -Hmlag zeros(k-1,T+2) -Hm zeros(k-1,1);dm -hmlag -hslag zeros(1,T+1) -hm -hs;zeros(1,k+q+1) -e_T1' e_ps' -1];
   C = [eye(q) zeros(q,T+1);zeros(k+1,T+q+1); zeros(T+1,q) eye(T+1); zeros(k+1,T+q+1)];
else;
   Ht = [];
   for i=1:NumLead;
      Hi = Hexp(:,1+((i-1)*k):i*k);
      Ht = [Ht [Kpm'*Hi zeros(k-1,1);e_pm'*Hi*Kps*Kps' e_pm'*Hi*e_ps]];
   end;
   %
   % define A, B and C
   %
   A = [eye(T+q+1+(NumLead*(k+1))) zeros(T+q+1+(NumLead*(k+1)),k+1);zeros(k,T+2+q+k) Ht;zeros(1,T+q+1+((NumLead+1)*(k+1)))];
   B = [zeros(q,T+q+1+((NumLead+1)*(k+1)));zeros(k+1,T+2+k+q) eye(k+1) zeros(k+1,(NumLead-1)*(k+1));zeros(T+1,q+k+1) M zeros(T+1,NumLead*(k+1));zeros((NumLead-1)*(k+1),T+q+3+(2*k)) eye((NumLead-1)*(k+1));Dm -Hmlag zeros(k-1,T+2) -Hm zeros(k-1,((NumLead-1)*(k+1))+1);dm -hmlag -hslag zeros(1,T+1) -hm -hs zeros(1,(NumLead-1)*(k+1));zeros(1,q+k+1) -e_T1' e_ps' -1 zeros(1,(NumLead-1)*(k+1))];
   C = [eye(q) zeros(q,T+1);zeros(k+1,T+q+1);zeros(T+1,q) eye(T+1);zeros(NumLead*(k+1),T+q+1)];
end;
kT = T+k+q+2;
lT = size(A,2);
%
% compute the QZ decomposition
%
[S,T,Q,Z] = qz(A,B);
if OrderQZ==1;
   [S,T,Q,Z] = ordqz(S,T,Q,Z,'udo');
else;
   [S,T,Q,Z] = YADAqzdiv(1+condn,S,T,Q,Z);
end;
Z11 = Z(1:kT,1:kT);
if rank(Z11)<kT;
   mcode = -1;
   return;
end;
%
% check for other errors
%
mcode = 1;
if abs(T(kT,kT))>abs(S(kT,kT))+condn;
   %
   % too many large eigenvalues
   %
   mcode = -2;
elseif abs(T(kT+1,kT+1))<abs(S(kT+1,kT+1));
   %
   % too few large eigenvalue
   %
   mcode = -3;
end;
%
% compute the transition matrix
%
S11 = S(1:kT,1:kT);
T11 = T(1:kT,1:kT);
%
% now X(t+1) = G*X(t) + C1*[eta(t+1)' Phi(t+1)']
%
G = real((Z11*(S11\T11))/Z11);
C1 = C(1:kT,:);
%
%  x(t) = P*X(t)
%
Z21 = Z(kT+1:lT,1:kT);
P = real(Z21/Z11);
%
% check the solution
%
% Gtilde = B(1:kT,1:kT)+(B(1:kT,kT+1:lT)*P);
%
% G-Gtilde
%
% APG = A(kT+1:lT,kT+1:lT)*P*G;
% BBP = B(kT+1:lT,1:kT) + (B(kT+1:lT,kT+1:lT)*P);
%
% APG-BBP
%
% these two differences should be close to zero
%

%
% end of ZLBKleinSolver.m
%
