% dsge_hess_adaptive.m

% sbsv is structural break with known date

% uses adaptive computation of the hessian matrix

% - computes the Hessian at the posterior mode
% - direct translation of professor Schorfheide's gauss program
% - Note that we don't do parameter transformation.
% - by Minchul Shin
clear hessian % to reserve for the function name

% -------------------------------------------------------------------------
% House cleaning and set defaults
% -------------------------------------------------------------------------
% set clock
time0 = now;

% start log file
whattimeisit = clock;
logfilename  = ['log_hess_adaptive', num2str(floor(whattimeisit(1,2:6))), '.log'];
cd(opath);
diary(logfilename);
cd(workpath);

% -------------------------------------------------------------------------
% Import prior specification
% -------------------------------------------------------------------------
% load prior
eval(['[prior, paraname] = prior_', specname,';'])

npara    = size(prior,1);
pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

% -------------------------------------------------------------------------
% Load Posterior Mode Estimates
% -> save('pm.mat', 'pmod', 'pmod_post', 'pmod_prior', 'retcodeh');
% -------------------------------------------------------------------------
% check if pm results exists and load if it exists
cd(lpath);

% check
if exist('pm.mat', 'file') == 0
    disp('Please run posterior maximization program first !!!');
    disp('Stop this program');
    pause
end

% load
load('pm.mat'); % load posterior mode
para = pmod;
cd(workpath);

% display loaded result
disp('------------------------------------------');
disp('Esimation Results (posterior mode)');
disp('------------------------------------------');
disp([{'parameters', 'mode'}; [paraname num2cell(fixdec(pmod,3))]]);
disp('------------------------------------------');

% -------------------------------------------------------------------------
% Compute "NEGATIVE" value of Hessian
%   - element by element, fine tune with dxscale
% -------------------------------------------------------------------------
% define function handle
fcn_dsgehess = @(x) objfcn_hess_adaptive_sbsv(x, prior, estinfo, YY, modelname, order_app, n_particle, T0_);


% An and Schorfheide's code (modified by Minchul)
% hssn     : hessian calculated by subset of fullhssn (simple average of them)
% errorij  : error control for correlation from this hssn to be withn 1
% fullhssn : hessians based on various step size
% [hssn, errorij, fullhssn] = nhess(fcn_dsgehess, pmod);

[hssn, err] = hessian(fcn_dsgehess, pmod(logical(pmaskinv),1));
hssn = trans_fix(hssn,prior);


% take negative hessian
X = -hssn;

%--------------------------------------------------------------------
% Variance: Pseudo-inverse of the negative of Hessian
%--------------------------------------------------------------------
% SVD
[U,S,V] = svd(X,0);
% [U,S,V] = svd(X);
s = diag(S);

% rank of Hessian (theoretical)
r1 = sum(pmaskinv);

% evaluate rank
tol = size(X,1)*max(s)*eps;
r2  = sum(s>tol);
r   = r1; % let's use theoretical rank

% inverse of negative Hessian
S(:) = 0;
if (r == 0)
    error('every singular value is zero.');
else
    S(1:r, 1:r) = diag(ones(r,1)./s(1:r));
    sigmult     = U*sqrt(S); %sigmult: square root of cov matrix at the posterior mode (this is well defined only when negative hess is positive definite)
    logdetS     = sum(log(diag(S(1:r,1:r))));
    penalt      = (r/2)*log(2*pi) + 0.5*logdetS; %posterior mode penalty
end

% compute std of mode estimates
pmod_std = sqrt(diag(sigmult*sigmult')); %sigmult: square root of cov matrix
invHHM   = sigmult*sigmult'; %inverse of negative hessian


% print results
disp('-----------------------------------------');
disp(['Model          ', modelname]);
disp(['Prior          ', lprior]);
disp(['Dataset        ', ldata]);
disp(['Order          ', num2str(order_app)]);
disp(['# of particles ', num2str(n_particle)]);
disp('------------------------------------------');
disp(['Det. of Cov matrix : ', num2str(fixdec(logdetS,100))]);
disp(['Post Mode Penalty  : ', num2str(fixdec(penalt,5))]);
disp('Log of -----------------------------------');
disp(['Posterior at Mode    : ', num2str(pmod_post)]);
disp(['Likelihood at Mode   : ', num2str(pmod_post-pmod_prior)]);
disp(['Prior at Mode        : ', num2str(pmod_prior)]);
disp('------------------------------------------');
disp(['Return code(cminwel) : ', num2str(retcodeh)]);
disp('------------------------------------------');
disp('Esimation Results');
disp('------------------------------------------');
disp([{'parameters', 'mode', 'S.D.'}; [paraname num2cell(fixdec(pmod,3)) num2cell(fixdec(pmod_std,4))]]);
disp('------------------------------------------');


% ------------------------------------------------------------------------
% Save outputs
% ------------------------------------------------------------------------
% hessian: negative hessian
% invHHm : inverse of negative hessian
% sigmult: sqrt of negative inverse hessian
% ------------------------------------------------------------------------
hessian = invHHM;

cd(opath);
save('hess_adaptive.mat', 'hessian', 'hssn', 'invHHM', 'sigmult', 'penalt', 'pmod', 'pmod_std', 'pmod_post', 'pmod_prior');
cd(workpath);

% ------------------------------------------------------------------------
% calculate elapsed time and print into log
% ------------------------------------------------------------------------
time1 = now;
disp(' ');
disp(' ');
disp('-------------------------------------------------------');
disp(['start       :  ' datestr(time0)]);
disp(['end         :  ' datestr(time1)]);
disp(['elapsed time:  ' datestr(time1-time0,13)]);
disp('-------------------------------------------------------');
disp(' ');
% close log file
diary off



